/****************************************************************************
 * (c) 2018 softwareentwicklung-als-prozess.de
 * Funktionalitaet:
 * Interne Spannung messen und als bitwert und Spannung in Volt auf einem 4 Zeilen LCD ausgeben.
 ***************************************************************************/
// Includes -----------------------------------------------------------------
#include "i2clcd.h"
#include "vctl.h"
#include "glaetten.h"

// Definitionen
#define TEXTLEN 21
#define LUX_CALC_SCALAR     12518931.0
#define LUX_CALC_EXPONENT   -1.405

// interne Prototypen
static void cleartext(void);
static double CalcLux(void);
static uint16_t get_Analog(void);

// globale Variablen
char     text[TEXTLEN];
uint32_t uref = 0l;
double   lux  = 0.0;
uint16_t tmp  = 0;
uint16_t frac = 0;
uint16_t bits = 0;

// Funktionen
void setup() 
{
  vctl_Setup();
  i2c_lcd_Setup();
}

void loop() 
{
  // LCD Helligkeit
  lux  = (uint32_t)CalcLux();
  tmp  = (uint16_t)lux;                                 // Vor komma
  frac = (uint16_t)(lux * 10.0) - ((uint16_t)lux * 10); // eine Stelle nach Komma
  sprintf(text, "Licht = %5u.%1u lux ", tmp,frac);  
  i2c_lcd_Write(I2CLCD_LINE1,text);
  cleartext();
  sprintf(text," bits = %-4d",bits);
  i2c_lcd_Write(I2CLCD_LINE2,text);
  cleartext();
  // LCD interne Spannung
  uref = vctl_Measure();              // interne Spannung messen (V_BG)
  sprintf(text,"  Vcc = %4i mV",(uint16_t)uref); // umwandeln in ASCII
  i2c_lcd_Write(I2CLCD_LINE3,text);
  cleartext();
  if (VCTL_VCCOK < uref)
  {
    sprintf(text,"  BAT = OK          ");
  }
  else
  {
    if (VCTL_VCCLOW < uref)
    {
      sprintf(text,"  BAT = niedrig     ");
    }
    else
    {
      sprintf(text,"  BAT = zu niedrig  ");
    }
  }
  i2c_lcd_Write(I2CLCD_LINE4,text);    
  cleartext();
}

// Textausgabefeld initialisieren
static void cleartext(void)
{
  int i = 0;

  for(i = 0; i < (TEXTLEN - 1); i++)
  {
    text[i] = EMPTY;
  }
  text[TEXTLEN - 1] = ENDE;
}

static uint16_t get_Analog(void)
{
  uint16_t result = (uint16_t)0;
  int adcl, adch;
  
  ADMUX |= 3;                    // Kanal 3 als Eingang
  ADCSRA |= (1 << ADEN);         // ADC enable
  ADCSRA |= (1 << ADSC);         // starten
  while (bitRead(ADCSRA, ADSC)); // bis Messung beendet ist
  {                              // ersten Messwert verwerfen
  }                              //  wegen umschaltung der Referenzspannung
  ADCSRA |= (1 << ADEN);         // ADC enable
  ADCSRA |= (1 << ADSC);         // starten
  while (bitRead(ADCSRA, ADSC)); // bis Messung beendet ist
  // Wichtig: zuerst ADCL auslesen, dann ADCH
  adcl = ADCL;
  adch = ADCH & 0x03;           // ADLAR ist standardmaessig auf 0
                                // also die oberen 6 bit loeschen
  result = (adch << 8) | adcl;
  smooth_CircBufU(result);      // Messwert glaetten
  return(result);
}

static double CalcLux(void)
{
  double rvolt, lvolt;
  double resist;
  double illumi;
  
  // Eingangs-Spannung am ADC vom Widerstand berechnen
//  bits = get_Analog();
  bits = analogRead(A3);
  rvolt = (double)bits / 1023.0 * ((double)uref / 1000.0);
  // Spannung am LDR berechnen
  lvolt = ((double)uref / 1000.0) - rvolt;
  // Widerstand des LDR im Spannungsteiler berechnen
  resist = lvolt / rvolt * 10000.0;
  // Beleuchtungsstaerke in lux berechnen    
  illumi = LUX_CALC_SCALAR * pow(resist, LUX_CALC_EXPONENT);
  return(illumi);
}
