/****************************************************************************
 * (c) 2019 softwareentwicklung-als-prozess.de
 * Funktionalitaet:
 * interne Spannungsueberwachung mit dem ADC
 ***************************************************************************/
#include <arduino.h>
#include "vctl.h"

// verfuegbar fuer folgende Prozessoren - immer nur einen setzen
//#define ATMEGA328
//#define ATMEGA2560
#define ATTINY25_45_85
//#define ANDERE

#ifdef ATMEGA328
void vctl_Setup(void)
{ 
  ADMUX |= (1 << REFS0);                             // VCC als Referenzspannung
  ADMUX |= (1 << MUX3) | (1 << MUX2) | (1 << MUX1);  // VBG als Eingang
  delay(10);                                         // warten bis sich die Referenzspannung eingestellt hat
  ADCSRA |= (1 << ADEN);                             // ADC enable
}
#endif
#ifdef ATMEGA2560
void vctl_Setup(void)
{ 
  ADMUX |= (1 << REFS0);                             // VCC als Referenzspannung
  ADMUX |= (1 << MUX4) | (1 << MUX3) | (1 << MUX2) | (1 << MUX1);  // VBG als Eingang
  delay(10);                                         // warten bis sich die Referenzspannung eingestellt hat
  ADCSRA |= (1 << ADEN);                             // ADC enable
}
#endif
#ifdef ATTINY25_45_85
void vctl_Setup(void)
{ 
  ADMUX = 0;                                         // VCC als Referenzspannung
  ADMUX |= (1 << MUX3) | (1 << MUX2);                // VBG als Eingang
  delay(10);                                         // warten bis sich die Referenzspannung eingestellt hat
  ADCSRA |= (1 << ADEN);                             // ADC enable
}
#endif

#ifdef ANDERE
void vctl_Setup(void)
{ 
// je nach Prozessor - siehe Datenblatt / Datasheet
}
#endif

#if defined(ATMEGA328) || defined(ATMEGA2560) || defined(ATTINY25_45_85)
long vctl_Measure(void)
{
  int adcl, adch;
  long result;
  
  vctl_Setup();                     // fuer jede Messung neu
  ADCSRA |= (1 << ADSC);            // starten
  while (bitRead(ADCSRA, ADSC));    // bis Messung beendet ist
  // Wichtig: zuerst ADCL auslesen, dann ADCH
  adcl = ADCL;
  adch = ADCH & 0x03;               // ADLAR ist standardmaessig auf 0
                                    // also die oberen 6 bit loeschen
  result = (adch << 8) | adcl;
  return(1125300L / result);        // Spannung in millivolt: 1100mV * 1023 = 1125300
}
#endif
#ifdef ANDERE
long vctl_Measure(void)
{
// je nach Prozessor - siehe Datenblatt / Datasheet
  return(0l);
}
#endif
